//
//  USBClientLib.h
//  CppUserClient
//
//  Copyright (C) 2021 WCH.
//  Created by WCH TECH<tech@wch.cn> on 2021/9/17.
//

#import <Foundation/Foundation.h>
#import <IOKit/usb/USB.h>

//index 设备序号
typedef void (^completionCallback)(NSInteger connection);
/*
 delegate
 */
@protocol CH37xLibDelegate <NSObject>

/*
 设备连接回调
 locationID 设备在USB接口的位置
 */
- (void)CH37xDeviceConnect:(NSString *)locationID;

/*
 设备断开回调
 locationID 设备在USB接口的位置
 */
- (void)CH37xDeviceDisConnect:(NSString *)locationID;

@end


@interface CH37xLib : NSObject

@property (nonatomic, weak)id <CH37xLibDelegate> delegate;

/*
 扫描设备
 */
- (NSInteger)CH37xScanDevices;

/*
 获取扫描到的设备
 */
- (NSArray *)CH37xGetAllDevices;

/*
 根据设备获取LocationID
 device 当前设备
 */
- (NSString *)CH37xGetLocationID:(NSString *)device;

/*
 打开设备
 */
- (IOReturn)CH37xOpenDevice:(NSInteger)index;

/*
 关闭设备
 */
- (IOReturn)CH37xCloseDevice:(NSInteger)index;

/*
 同步读取
 index 设备序号
 bEndpointAddress 端点地址
 buffer 用于接收数据的buffer
 size 读取的数据长度
 */
- (IOReturn)CH37xSyncReadData:(NSInteger)index bEndpointAddress:(uint8_t)bEndpointAddress buffer:(void *)buffer size:(size_t)size;

/*
 中断读取
 index 设备序号
 bEndpointAddress 端点地址
 buffer 用于接收数据的buffer
 size 读取的数据长度
 */
- (IOReturn)CH37xInterruptRead:(NSInteger)index bEndpointAddress:(uint8_t)bEndpointAddress buffer:(void *)buffer size:(size_t)size;

/*
 设置上传缓冲区大小，驱动内部默认100M
 index 设备序号
 bufferSize 缓冲区大小
 */
- (IOReturn)CH37xSetBufUpload:(NSInteger)index bufferSize:(size_t)bufferSize;

/*
 开始缓冲区上传
 index 设备序号
 bEndpointAddress 端点地址
 packageSize 每包数据大小，驱动默认每包1M大小
 */
- (IOReturn)CH37xStartBufUpload:(NSInteger)index bEndpointAddress:(uint8_t)bEndpointAddress packageSize:(size_t)packageSize;

/*
 从上传缓冲区读取数据包
 返回值：实际读取的数据长度
 index 设备序号
 bEndpointAddress 端点地址
 buffer 用于接收数据的buffer
 size 读取的包长度，按照CH37xStartBufUpload包大小进行取
 */
- (IOReturn)CH37xReadEndP:(NSInteger)index bEndpointAddress:(uint8_t)bEndpointAddress buffer:(uint8_t *)buffer size:(size_t)size;


/*
 停止读取数据块
 index 设备序号
 bEndpointAddress 端点地址
 */
- (IOReturn)CH37xStopBufUpload:(NSInteger)index bEndpointAddress:(uint8_t)bEndpointAddress;


/*
 获取选择端点缓冲区数据包的个数
 返回值：buffer中数据包个数
 index 设备序号
 bEndpointAddress 端点地址
 */
- (NSInteger)CH37xQueryBuf:(NSInteger)index bEndpointAddress:(uint8_t)bEndpointAddress;

/*
 同步写入
 index 设备序号
 bEndpointAddress 端点地址
 buffer 写入的数据内容
 size 写入的数据长度
 */
- (IOReturn)CH37xSyncWriteData:(NSInteger)index bEndpointAddress:(uint8_t)bEndpointAddress buffer:(void *)buffer size:(size_t)size;


/*
 设置下传缓冲区大小，驱动内部默认100M
 index 设备序号
 bufferSize 缓冲区大小
 */
- (IOReturn)CH37xSetBufDownload:(NSInteger)index bufferSize:(size_t)bufferSize;

/*
 重置写端点的状态
 index 设备序号
 bEndpointAddress 端点地址
 */
- (IOReturn)CH37xResetWrite:(NSInteger)index bEndpointAddress:(uint8_t)bEndpointAddress;

/*
 缓冲区下传
 index 设备序号
 bEndpointAddress 端点地址
 size 写入的数据长度
 buffer 写入的数据内容
 isRead 是否需要进行写时读(如果业务需要写时，进行读，请开启)
 */
- (IOReturn)CH37xStartBufDownload:(NSInteger)index bEndpointAddress:(uint8_t)bEndpointAddress buffer:(void *)buffer size:(size_t)size needRead:(BOOL)isRead;

/*
 设置USB读写超时
 index 设备序号
 readTimeout 读的超时时间ms
 writeTimeout 写的超时时间ms
 */
- (IOReturn)CH37xSetTimeout:(NSInteger)index readTimeout:(uint64_t)readTimeout writeTimeout:(uint64_t)writeTimeout;

/*
 获取驱动版本号
 size buffer长度限制为64
 buffer 保存设备的版本号信息
 */
- (IOReturn)CH37xGetVersion:(NSInteger)index buffer:(void *)buffer size:(size_t)size;

/*
 获取设备ID
 index 设备序号
 buffer 保存返回值的
 buffer[0] = idProdcut
 buffer[1] = idVendor
 */
- (IOReturn)CH37xGetDeviceID:(NSInteger)index buffer:(uint64_t *)buffer size:(uint32_t)size;

/*
 获取所有端点
 index 设备序号
 返回值：保存IOUSBEndpointDescriptor的数组
 enumerateObjectsUsingBlock 方法进行枚举
 */
- (NSArray *)CH37xGetDeviceMsg:(NSInteger)index;

/*
 获取Bulk In端点
 index 设备序号
 返回值：保存IOUSBEndpointDescriptor的数组
 enumerateObjectsUsingBlock 方法进行枚举
 */
- (NSArray *)CH37xGetBulkInEndPoint:(NSInteger)index;

/*
 获取Bulk Out端点
 index 设备序号
 返回值：保存IOUSBEndpointDescriptor的数组
 enumerateObjectsUsingBlock 方法进行枚举
 */
- (NSArray *)CH37xGetBulkOutEndPoint:(NSInteger)index;

/*
 获取Interrupt端点
 index 设备序号
 返回值：保存IOUSBEndpointDescriptor的数组
 enumerateObjectsUsingBlock 方法进行枚举
 */
- (NSArray *)CH37xGetInterrputEndPoint:(NSInteger)index;

/*
 获取设备描述
 index 设备序号
 */
- (IOReturn)CH37xGetDeviceDesc:(NSInteger)index buffer:(uint64_t *)buffer;


@end

